@@ -1,12 +1,12 @@
-from django.db import models
-
+import os
+import datetime
 from logging import getLogger
 from shutil import rmtree
 
-from pki.openssl import OpensslActions, OpensslCaManagement, md5_constructor, refresh_pki_metadata
-from pki.settings import ADMIN_MEDIA_PREFIX, MEDIA_URL, PKI_BASE_URL, PKI_DEFAULT_COUNTRY, PKI_ENABLE_GRAPHVIZ
+from django.db import models
 
-import datetime, os
+from pki.openssl import OpensslActions, md5_constructor, refresh_pki_metadata
+from pki.settings import ADMIN_MEDIA_PREFIX, MEDIA_URL, PKI_BASE_URL, PKI_DEFAULT_COUNTRY, PKI_ENABLE_GRAPHVIZ, PKI_ENABLE_EMAIL
 
 logger = getLogger("pki")
 
@@ -62,7 +62,7 @@ COUNTRY    = ( ('AD', 'AD'),('AE', 'AE'),('AF', 'AF'),('AG', 'AG'),('AI', 'AI'),
 ##------------------------------------------------------------------##
 
 class CertificateBase(models.Model):
-    '''Base class for all type of certificates'''
+    """Base class for all type of certificates"""
     
     description  = models.CharField(max_length=255)
     country      = models.CharField(max_length=2, choices=COUNTRY, default='%s' % PKI_DEFAULT_COUNTRY.upper() )
@@ -90,15 +90,37 @@ class CertificateBase(models.Model):
     class Meta:
         abstract = True
     
+    ##------------------------------------------------------------------##
+    ## Helper functions
+    ##------------------------------------------------------------------##
+    
     def get_icon_html(self, state):
+        """Return HTML based on state.
+        
+        True : Return Django's yes icon
+        False: Return Django's no icon
+        """
         
         if state is True:
             return '<center><img src="%simg/admin/icon-yes.gif" alt="True" /></center>' % ADMIN_MEDIA_PREFIX
         else:
             return '<center><img src="%simg/admin/icon-no.gif" alt="False" /></center>'  % ADMIN_MEDIA_PREFIX
     
+    def get_pki_icon_html(self, img, alt="", title=""):
+        """Return HTML for given image.
+        
+        Can add optional alt and title parameters.
+        """
+        
+        img_path = os.path.join(PKI_BASE_URL, MEDIA_URL, 'pki/img', img)
+        return '<img src="%s" alt="%s" title="%s"/>' % (img_path, alt, title)
+    
+    ##------------------------------------------------------------------##
+    ## Changelist list_display functions
+    ##------------------------------------------------------------------##
+    
     def active_center(self):
-        '''Overwrite the Booleanfield admin for admin's changelist'''
+        """Overwrite the Booleanfield admin for admin's changelist"""
         
         return self.get_icon_html(self.active)
     
@@ -106,7 +128,21 @@ class CertificateBase(models.Model):
     active_center.short_description = 'Active'
     active_center.admin_order_field = 'active'
     
+    def Serial_align_right(self):
+        """Make serial in changelist right justified"""
+        
+        return '<div style="text-align:right;">%s</div>' % self.serial
+    
+    Serial_align_right.allow_tags = True
+    Serial_align_right.short_description = 'Serial'
+    Serial_align_right.admin_order_field = 'serial'
+    
     def Description(self):
+        """Limit description for changelist.
+        
+        Limit description to 30 characters to make changelist stay on one line per item.
+        At least in most cases.
+        """
         
         if len(self.description) > 30:
             return "%s..." % self.description[:30]
@@ -117,6 +153,13 @@ class CertificateBase(models.Model):
     Description.admin_order_field = 'description'
     
     def Expiry_date(self):
+        """Return expiry date with days left.
+        
+        Return color marked expiry date based on days left.
+        < 0 : EXPIRED text and red font color
+        < 30: Orange color
+        > 30: Just date and days left
+        """
         now = datetime.datetime.now().date()        
         diff = self.expiry_date - now
         
@@ -130,26 +173,12 @@ class CertificateBase(models.Model):
     Expiry_date.allow_tags = True
     Expiry_date.admin_order_field = 'expiry_date'
     
-    def CA_chain(self):
-        return "%s" % self.ca_chain
-    
-    CA_chain.allow_tags = True
-    CA_chain.admin_order_field = 'ca_chain'
-    
-    def build_download_links(self, type):
+    def Chain_link(self):
+        """Display chain link.
         
-        items_on_active = ( 'pem', 'key', 'chain' )
-        result = []
-        
-        for i in items_on_active:
-            if self.active:
-                result.append( '<a href="%s/pki/download/%s/%d/%s"><strong>%s</strong></a>' % (PKI_BASE_URL, type, self.id, i, i) )
-            else:
-                result.append( '<font color="grey">%s</font>' % i )
-        
-        return result
-    
-    def Locate_link(self):
+        If PKI_ENABLE_GRAPHVIZ is True a colored chain link is displayed. Otherwise
+        a b/w chain icon without link is displayed.
+        """
         
         type = "cert"
         
@@ -157,19 +186,79 @@ class CertificateBase(models.Model):
             type = "ca"
         
         if PKI_ENABLE_GRAPHVIZ:
-            return '<center><a href="%s/pki/locate/%s/%d" target="_blank"><img src="%s/pki/img/mag.png" height="13px" width="13px" alt="Locate" title="Locate object in PKI tree"/></a></center>' % (PKI_BASE_URL, type, self.pk, os.path.join(PKI_BASE_URL, MEDIA_URL))
+            return '<center><a href="%s/pki/chain/%s/%d" target="_blank">%s</a></center>' % (PKI_BASE_URL, type, self.pk, self.get_pki_icon_html('chain.png', "Show chain", "Show object chain"))
         else:
-            return '<center><img src="%s/pki/img/mag_disabled.png" alt="Locate" title="Enable setting PKI_ENABLE_GRAPHVIZ"/></center>' % os.path.join(PKI_BASE_URL, MEDIA_URL)
+            return '<center>%s</center>' % self.get_pki_icon_html("chain.png", "Show chain", "Enable setting PKI_ENABLE_GRAPHVIZ")
     
-    Locate_link.allow_tags = True
-    Locate_link.short_description = 'Find'
+    Chain_link.allow_tags = True
+    Chain_link.short_description = 'Chain'
+    
+    def Email_link(self):
+        """Display email link based on status.
+        
+        If PKI_ENABLE_EMAIL or certificate isn't active a disabled (b/w) icon is displayed.
+        If no email address is set in the certificate a icon with exclamation mark is displayed.
+        Otherwise the normal icon is returned.
+        """
+        
+        if not PKI_ENABLE_EMAIL:
+            result  = '<center>%s</center>' % self.get_pki_icon_html("mail--arrow_bw.png", "Send email", "Enable setting PKI_ENABLE_EMAIL")
+        elif not self.active:
+            result  = '<center>%s</center>' % self.get_pki_icon_html("mail--arrow_bw.png", "Send email", "Certificate is revoked. Disabled")
+        else:
+            type = "cert"
+            
+            if self.__class__.__name__ == "CertificateAuthority": type = "ca"
+            
+            if self.email:
+                result  = '<center><a href="%s/pki/email/%s/%d">%s</a></center>' % (PKI_BASE_URL, type, self.pk, self.get_pki_icon_html("mail--arrow.png", "Send email", "Send cert to specified email"))
+            else:
+                result  = '<center>%s</center>' % self.get_pki_icon_html("mail--exclamation.png", "Send email", "Certificate has no email set. Disabled")
+        
+        return result
+    
+    Email_link.allow_tags = True
+    Email_link.short_description = 'Email'
+    
+    def Download_link(self):
+        """Return a download icon.
+        
+        Based on object status => clickable icon or just a b/w image
+        """
+        
+        if self.active:
+            type = "cert"
+            
+            if self.__class__.__name__ == "CertificateAuthority": type = "ca"
+            
+            return '<center><a href="%s/pki/download/%s/%d/">%s ZIP</href></center></center>' % (PKI_BASE_URL, type, self.pk, \
+                                                                                                 self.get_pki_icon_html("drive-download.png", "Download", "Download certificate data"))
+        else:
+            return '<center>%s <font color="grey">ZIP</font></center>' % self.get_pki_icon_html("drive-download_bw.png", "Download", "Cannot download because certificate is revoked")
+    
+    Download_link.allow_tags = True
+    Download_link.short_description = 'Download'
+    
+    def Parent_link(self):
+        """Return parent name.
+        
+        Returns parent's name when parent != None or self-signed
+        """
+        
+        if self.parent:
+            return '<a href="../certificateauthority/%d/" style="font-weight: bold;">%s</a>' % (self.parent.pk, self.parent.common_name)
+        else:
+            return "self-signed"
+    
+    Parent_link.allow_tags = True
+    Parent_link.short_description = 'Parent'
     
 ##------------------------------------------------------------------##
 ## Certificate authority class
 ##------------------------------------------------------------------##
 
 class CertificateAuthority(CertificateBase):
-    '''CA specification'''
+    """Certificate Authority model"""
     
     ##---------------------------------##
     ## Model definition
@@ -194,17 +283,21 @@ class CertificateAuthority(CertificateBase):
         verbose_name_plural = 'Certificate Authorities'
         permissions         = ( ( "can_download_ca", "Can download", ), )
     
+    def __unicode__(self):
+        return self.name
+    
     ##---------------------------------##
     ## Redefined functions
     ##---------------------------------##
     
     def save(self, force_insert=False, force_update=False):
+        """Save the CertificateAuthority object"""
         
         if self.pk:
             ### existing CA
             if self.action in ('update', 'revoke', 'renew'):
                 
-                action = OpensslActions('ca', self)
+                action = OpensslActions(self)
                 prev   = CertificateAuthority.objects.get(pk=self.pk)
                 
                 if self.action == 'update':
@@ -317,7 +410,7 @@ class CertificateAuthority(CertificateBase):
             self.rebuild_ca_metadata(modify=True, task='append')
             
             ## Generate keys and certificates
-            action = OpensslActions('ca', self)
+            action = OpensslActions(self)
             action.generate_key()
             
             if not self.parent:
@@ -347,9 +440,9 @@ class CertificateAuthority(CertificateBase):
             if self.parent == None:
                 chain.append('self-signed')
             else:
-                chain.append( self.name )
+                chain.append( self.common_name )
                 while p != None:
-                    chain.append(p.name)
+                    chain.append(p.common_name)
                     p = p.parent
             
             chain.reverse()
@@ -359,7 +452,7 @@ class CertificateAuthority(CertificateBase):
                 if chain_str == '':
                     chain_str += '%s' % i
                 else:
-                    chain_str += '&rarr;%s' % i
+                    chain_str += ' &rarr; %s' % i
             
             self.ca_chain = chain_str
             
@@ -373,6 +466,7 @@ class CertificateAuthority(CertificateBase):
         super(CertificateAuthority, self).save()
     
     def delete(self, passphrase):
+        """Delete the CertificateAuthority object"""
         
         logger.info( "Certificate %s is going to be deleted" % self.name )
         
@@ -404,7 +498,7 @@ class CertificateAuthority(CertificateBase):
         
         ## Remoke first ca in the chain
         if revoke_required:
-            c_action = OpensslActions('ca', CertificateAuthority.objects.get(pk=self.pk))
+            c_action = OpensslActions(CertificateAuthority.objects.get(pk=self.pk))
             c_action.revoke_certificate(passphrase)
             c_action.generate_crl(ca=self.parent.name, pf=passphrase)
         
@@ -419,9 +513,7 @@ class CertificateAuthority(CertificateBase):
     ##---------------------------------##
     
     def rebuild_ca_metadata(self, modify, task):
-        '''Wrapper around refresh_pki_metadata. Following the DRY principle'''
-        
-        logger.info("Rebuilding CA store")
+        """Wrapper around refresh_pki_metadata"""
         
         if modify:
             if task == 'append':
@@ -443,27 +535,6 @@ class CertificateAuthority(CertificateBase):
     ## View functions
     ##---------------------------------##
     
-    def __unicode__(self):
-        return self.name
-    
-    def download(self):
-        
-        items = self.build_download_links('ca')
-        
-        if self.active:
-            if self.der_encoded:
-                items.append( '<a href="%s/pki/download/ca/%d/der"><strong>der</strong></a>' % (PKI_BASE_URL, self.pk) )
-            else:
-                items.append( '<font color="grey">der</font>' )
-            items.append( '<a href="%s/pki/download/ca/%d/crl"><strong>crl</strong></a>' % (PKI_BASE_URL, self.pk) )
-        else:
-            items.append( '<font color="grey">der</font>' )
-            items.append( '<font color="grey">crl</font>' )
-        
-        return ' | '.join(items)
-    
-    download.allow_tags = True
-    
     def Tree_link(self):
         
         if PKI_ENABLE_GRAPHVIZ:
@@ -473,14 +544,13 @@ class CertificateAuthority(CertificateBase):
     
     Tree_link.allow_tags = True
     Tree_link.short_description = 'Tree'
-
-    
+  
 ##------------------------------------------------------------------##
 ## Certificate class
 ##------------------------------------------------------------------##
 
 class Certificate(CertificateBase):
-    '''CA specification'''
+    """Certificate model"""
     
     common_name       = models.CharField(max_length=64)
     name              = models.CharField(max_length=64, help_text="Only change the suggestion if you really know what you're doing")
@@ -500,17 +570,20 @@ class Certificate(CertificateBase):
         permissions         = ( ( "can_download", "Can download certificate", ), )
         unique_together     = ( ( "name", "parent" ), ("common_name", "parent"), )
     
+    def __unicode__(self):
+        return self.name
+    
     ##---------------------------------##
     ## Redefined functions
     ##---------------------------------##
     
     def save(self):
+        """Save the Certificate object"""
         
         if self.pk:
-            
             if self.action in ('update', 'revoke', 'renew'):
                 
-                action = OpensslActions('cert', self)
+                action = OpensslActions(self)
                 prev   = Certificate.objects.get(pk=self.pk)
                 
                 if self.action == 'update':
@@ -522,9 +595,9 @@ class Certificate(CertificateBase):
                         action.remove_der_encoded()
                     
                     ## Create or remove PKCS12 certificate
-                    if self.pkcs12_encoded and not prev.pkcs12_encoded:
-                        if prev.pkcs12_passphrase == self.pkcs12_passphrase:
-                            logger.debug( 'P12 passphrase is unchanged. Nothing to do' )
+                    if self.pkcs12_encoded:
+                        if prev.pkcs12_encoded and prev.pkcs12_passphrase == self.pkcs12_passphrase:
+                            logger.debug( 'PKCS12 passphrase is unchanged. Nothing to do' )
                         else:
                             action.generate_pkcs12_encoded()
                     else:
@@ -600,7 +673,7 @@ class Certificate(CertificateBase):
             logger.info( "***** { New certificate generation: %s } *****" % self.name )
             
             ## Generate key and certificate
-            action = OpensslActions('cert', self)
+            action = OpensslActions(self)
             
             action.generate_key()
             action.generate_csr()
@@ -640,50 +713,18 @@ class Certificate(CertificateBase):
             super(Certificate, self).save()
     
     def delete(self, passphrase):
-        
-        logger.info( "Certificate %s is going to be deleted" % self.name )
+        """Delete the Certificate object"""
         
         ## Remoke first ca in the chain
-        c_action = OpensslActions('cert', Certificate.objects.get(pk=self.pk))
+        c_action = OpensslActions(Certificate.objects.get(pk=self.pk))
         c_action.revoke_certificate(passphrase)
         c_action.generate_crl(ca=self.parent.name, pf=passphrase)
         
         ## Time for some rm action
-        a = OpensslActions('cert', self)
+        a = OpensslActions(self)
         a.remove_complete_certificate()
         
         ## Call the "real" delete function
         super(Certificate, self).delete()
 
-    ##---------------------------------##
-    ## View functions
-    ##---------------------------------##
-    
-    def __unicode__(self):
-        return self.name
-    
-    def download(self):
-        
-        items = self.build_download_links('cert')
-        
-        if self.active:
-            if self.der_encoded:
-                items.append( '<a href="%s/pki/download/cert/%d/der"><strong>der</strong></a>' % (PKI_BASE_URL, self.pk) )
-            else:
-                items.append( '<font color="grey">der</font>' )
-            
-            if self.pkcs12_encoded:
-                items.append( '<a href="%s/pki/download/cert/%d/pkcs12"><strong>p12</strong></a>' % (PKI_BASE_URL, self.pk) )
-            else:
-                items.append( '<font color="grey">p12</font>' )
-            
-            items.append( '<a href="%s/pki/download/cert/%d/csr"><strong>csr</strong></a>' % (PKI_BASE_URL, self.pk) )
-        else:
-            items.append( '<font color="grey">der</font>' )
-            items.append( '<font color="grey">p12</font>' )
-            items.append( '<font color="grey">csr</font>' )
-        
-        return ' | '.join(items)
-    
-    download.allow_tags = True
     
